unit TB2ExtItems;

{
  Toolbar2000
  Copyright (C) 1998-2001 by Jordan Russell
  All rights reserved.
  For conditions of distribution and use, see LICENSE.TXT.

  $Id: TB2ExtItems.pas,v 1.34 2001/09/13 02:48:35 jr Exp $
}

interface

{$I TB2Ver.inc}

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, CommCtrl, Menus, ActnList,
  TB2Item;

type
  TTBEditItemOption = (tboUseEditWhenVertical);
  TTBEditItemOptions = set of TTBEditItemOption;

const
  EditItemDefaultEditOptions = [];
  EditItemDefaultEditWidth = 64;

type
  TTBAcceptTextEvent = procedure(Sender: TObject; var NewText: String;
    var Accept: Boolean) of object;

  TTBEditAction = class(TAction)
  private
    FEditOptions: TTBEditItemOptions;
    FEditCaption: String;
    FEditWidth: Integer;
    FOnAcceptText: TTBAcceptTextEvent;
    FText: String;
    procedure SetEditCaption (Value: String);
    procedure SetEditOptions (Value: TTBEditItemOptions);
    procedure SetEditWidth (Value: Integer);
    procedure SetOnAcceptText (Value: TTBAcceptTextEvent);
    procedure SetText (Value: String);
  public
    constructor Create (AOwner: TComponent); override;
  published
    property EditCaption: String read FEditCaption write SetEditCaption;
    property EditOptions: TTBEditItemOptions read FEditOptions write SetEditOptions default EditItemDefaultEditOptions;
    property EditWidth: Integer read FEditWidth write SetEditWidth default EditItemDefaultEditWidth;
    property Text: String read FText write SetText;

    property OnAcceptText: TTBAcceptTextEvent read FOnAcceptText write SetOnAcceptText;
  end;

  TTBEditItemActionLink = class(TTBCustomItemActionLink)
  protected
    procedure AssignClient (AClient: TObject); override;
    function IsEditCaptionLinked: Boolean; virtual;
    function IsEditOptionsLinked: Boolean; virtual;
    function IsEditWidthLinked: Boolean; virtual;
    function IsOnAcceptTextLinked: Boolean; virtual;
    function IsTextLinked: Boolean; virtual;
    procedure SetEditCaption (const Value: String); virtual;
    procedure SetEditOptions (Value: TTBEditItemOptions); virtual;
    procedure SetEditWidth (const Value: Integer); virtual;
    procedure SetOnAcceptText (Value: TTBAcceptTextEvent); virtual;
    procedure SetText (const Value: String); virtual;
  end;

  TTBEditItem = class(TTBCustomItem)
  private
    FEditCaption: String;
    FEditOptions: TTBEditItemOptions;
    FEditWidth: Integer;
    FMaxLength: Integer;
    FOnAcceptText: TTBAcceptTextEvent;
    FText: String;
    function IsEditCaptionStored: Boolean;
    function IsEditOptionsStored: Boolean;
    function IsEditWidthStored: Boolean;
    function IsTextStored: Boolean;
    procedure SetEditCaption (Value: String);
    procedure SetEditOptions (Value: TTBEditItemOptions);
    procedure SetEditWidth (Value: Integer);
    procedure SetMaxLength (Value: Integer);
    procedure SetText (Value: String);
  protected
    procedure ActionChange (Sender: TObject; CheckDefaults: Boolean); override;
    function GetActionLinkClass: TTBCustomItemActionLinkClass; override;
    function GetItemViewerClass (AView: TTBView): TTBItemViewerClass; override;
    function NeedToRecreateViewer (AViewer: TTBItemViewer): Boolean; override;
  public
    constructor Create (AOwner: TComponent); override;
    procedure Clear;
    procedure Click; override;
  published
    property Action;
    property AutoCheck;
    property Caption;
    property Checked;
    property DisplayMode;
    property EditCaption: String read FEditCaption write SetEditCaption stored IsEditCaptionStored;
    property EditOptions: TTBEditItemOptions read FEditOptions write SetEditOptions stored IsEditOptionsStored;
    property EditWidth: Integer read FEditWidth write SetEditWidth stored IsEditWidthStored;
    property MaxLength: Integer read FMaxLength write SetMaxLength default 0;
    property Enabled;
    property GroupIndex;
    property HelpContext;
    property Hint;
    property ImageIndex;
    property ShortCut;
    property Text: String read FText write SetText stored IsTextStored;
    property Visible;

    property OnAcceptText: TTBAcceptTextEvent read FOnAcceptText write FOnAcceptText;
    property OnClick;
  end;

  TTBEditItemViewer = class(TTBItemViewer)
  private
    FEditControl: TEdit;
    FEditControlStatus: set of (ecsContinueLoop, ecsAccept, ecsClose);
    procedure BeginEdit (var CancelLoop: Boolean);
    function EditLoop (const CapHandle: HWND): Boolean;
    procedure EditWndProc (var Message: TMessage);
    procedure GetEditRect (var R: TRect);
  protected
    procedure CalcSize (const Canvas: TCanvas; var AWidth, AHeight: Integer);
      override;
    function CaptionShown: Boolean; override;
    procedure Draw (const Canvas: TCanvas; const ClientAreaRect: TRect;
      IsSelected, IsPushed, UseMenuColor: Boolean); override;
    function GetCaptionText: String; override;
    procedure GetCursor (const Pt: TPoint; var ACursor: HCURSOR); override;
    procedure KeyDown (var Key: Word; Shift: TShiftState;
      var EventData: TTBItemEventData); override;
    procedure MouseDown (Shift: TShiftState; X, Y: Integer;
      var EventData: TTBItemEventData); override;
    procedure MouseUp (X, Y: Integer; var EventData: TTBItemEventData);
      override;
  end;


implementation

uses
  TB2Common, TB2Consts;

const
  EditMenuTextMargin = 3;
  EditMenuMidWidth = 4;

type
  TEditAccess = class(TEdit);


{ TTBEditAction }

constructor TTBEditAction.Create (AOwner: TComponent);
begin
  inherited;
  FEditOptions := EditItemDefaultEditOptions;
  FEditWidth := EditItemDefaultEditWidth;
  DisableIfNoHandler := False;
end;

procedure TTBEditAction.SetEditCaption (Value: String);
var
  I: Integer;
begin
  if FEditCaption <> Value then begin
    for I := 0 to FClients.Count - 1 do
      if TBasicActionLink(FClients[I]) is TTBEditItemActionLink then
        TTBEditItemActionLink(FClients[I]).SetEditCaption (Value);
    FEditCaption := Value;
    Change;
  end;
end;

procedure TTBEditAction.SetEditOptions (Value: TTBEditItemOptions);
var
  I: Integer;
begin
  if FEditOptions <> Value then begin
    for I := 0 to FClients.Count - 1 do
      if TBasicActionLink(FClients[I]) is TTBEditItemActionLink then
        TTBEditItemActionLink(FClients[I]).SetEditOptions (Value);
    FEditOptions := Value;
    Change;
  end;
end;

procedure TTBEditAction.SetEditWidth (Value: Integer);
var
  I: Integer;
begin
  if FEditWidth <> Value then begin
    for I := 0 to FClients.Count - 1 do
      if TBasicActionLink(FClients[I]) is TTBEditItemActionLink then
        TTBEditItemActionLink(FClients[I]).SetEditWidth (Value);
    FEditWidth := Value;
    Change;
  end;
end;

procedure TTBEditAction.SetOnAcceptText (Value: TTBAcceptTextEvent);
var
  I: Integer;
begin
  if not MethodsEqual(TMethod(FOnAcceptText), TMethod(Value)) then begin
    for I := 0 to FClients.Count - 1 do
      if TBasicActionLink(FClients[I]) is TTBEditItemActionLink then
        TTBEditItemActionLink(FClients[I]).SetOnAcceptText (Value);
    FOnAcceptText := Value;
    Change;
  end;
end;

procedure TTBEditAction.SetText (Value: String);
var
  I: Integer;
begin
  if FText <> Value then begin
    for I := 0 to FClients.Count - 1 do
      if TBasicActionLink(FClients[I]) is TTBEditItemActionLink then
        TTBEditItemActionLink(FClients[I]).SetText (Value);
    FText := Value;
    Change;
  end;
end;


{ TTBEditItemActionLink }

procedure TTBEditItemActionLink.AssignClient (AClient: TObject);
begin
  FClient := AClient as TTBEditItem;
end;

function TTBEditItemActionLink.IsEditCaptionLinked: Boolean;
begin
  if Action is TTBEditAction then
    Result := TTBEditItem(FClient).EditCaption = TTBEditAction(Action).EditCaption
  else
    Result := False;
end;

function TTBEditItemActionLink.IsEditOptionsLinked: Boolean;
begin
  if Action is TTBEditAction then
    Result := TTBEditItem(FClient).EditOptions = TTBEditAction(Action).EditOptions
  else
    Result := False;
end;

function TTBEditItemActionLink.IsEditWidthLinked: Boolean;
begin
  if Action is TTBEditAction then
    Result := TTBEditItem(FClient).EditWidth = TTBEditAction(Action).EditWidth
  else
    Result := False;
end;

function TTBEditItemActionLink.IsOnAcceptTextLinked: Boolean;
begin
  if Action is TTBEditAction then
    Result := MethodsEqual(TMethod(TTBEditItem(FClient).OnAcceptText),
      TMethod(TTBEditAction(Action).OnAcceptText))
  else
    Result := False;
end;

function TTBEditItemActionLink.IsTextLinked: Boolean;
begin
  if Action is TTBEditAction then
    Result := TTBEditItem(FClient).Text = TTBEditAction(Action).Text
  else
    Result := False;
end;

procedure TTBEditItemActionLink.SetEditCaption (const Value: String);
begin
  if IsEditCaptionLinked then TTBEditItem(FClient).EditCaption := Value;
end;

procedure TTBEditItemActionLink.SetEditOptions (Value: TTBEditItemOptions);
begin
  if IsEditOptionsLinked then TTBEditItem(FClient).EditOptions := Value;
end;

procedure TTBEditItemActionLink.SetEditWidth (const Value: Integer);
begin
  if IsEditWidthLinked then TTBEditItem(FClient).EditWidth := Value;
end;

procedure TTBEditItemActionLink.SetOnAcceptText (Value: TTBAcceptTextEvent);
begin
  if IsOnAcceptTextLinked then TTBEditItem(FClient).OnAcceptText := Value;
end;

procedure TTBEditItemActionLink.SetText (const Value: String);
begin
  if IsTextLinked then TTBEditItem(FClient).Text := Value;
end;


{ TTBEditItem }

constructor TTBEditItem.Create (AOwner: TComponent);
begin
  inherited;
  FEditOptions := EditItemDefaultEditOptions;
  FEditWidth := EditItemDefaultEditWidth;
end;

procedure TTBEditItem.ActionChange (Sender: TObject; CheckDefaults: Boolean);
begin
  inherited;
  if Action is TTBEditAction then
    with TTBEditAction(Sender) do
    begin
      if not CheckDefaults or (Self.EditCaption = '') then
        Self.EditCaption := EditCaption;
      if not CheckDefaults or (Self.EditOptions = []) then
        Self.EditOptions := EditOptions;
      if not CheckDefaults or (Self.Text = '') then
        Self.Text := Text;
      if not CheckDefaults or not Assigned(Self.OnAcceptText) then
        Self.OnAcceptText := OnAcceptText;
    end;
end;

function TTBEditItem.GetActionLinkClass: TTBCustomItemActionLinkClass;
begin
  Result := TTBEditItemActionLink;
end;

function TTBEditItem.GetItemViewerClass (AView: TTBView): TTBItemViewerClass;
begin
  if not(tboUseEditWhenVertical in EditOptions) and
     (AView.Orientation = tbvoVertical) then
    Result := inherited GetItemViewerClass(AView)
  else
    Result := TTBEditItemViewer;
end;

function TTBEditItem.NeedToRecreateViewer (AViewer: TTBItemViewer): Boolean;
begin
  Result := GetItemViewerClass(AViewer.View) <> AViewer.ClassType;
end;

procedure TTBEditItem.Clear;
begin
  Text := '';
end;

procedure TTBEditItem.Click;
begin
  inherited;
end;

function TTBEditItem.IsEditOptionsStored: Boolean;
begin
  Result := (EditOptions <> EditItemDefaultEditOptions) and
    ((ActionLink = nil) or not(ActionLink is TTBEditItemActionLink) or
     not TTBEditItemActionLink(ActionLink).IsEditOptionsLinked);
end;

function TTBEditItem.IsEditCaptionStored: Boolean;
begin
  Result := (ActionLink = nil) or not(ActionLink is TTBEditItemActionLink) or
    not TTBEditItemActionLink(ActionLink).IsEditCaptionLinked;
end;

function TTBEditItem.IsEditWidthStored: Boolean;
begin
  Result := (EditWidth <> EditItemDefaultEditWidth) and
    ((ActionLink = nil) or not(ActionLink is TTBEditItemActionLink) or
     not TTBEditItemActionLink(ActionLink).IsEditWidthLinked);
end;

function TTBEditItem.IsTextStored: Boolean;
begin
  Result := (ActionLink = nil) or not(ActionLink is TTBEditItemActionLink) or
    not TTBEditItemActionLink(ActionLink).IsTextLinked;
end;

procedure TTBEditItem.SetEditOptions (Value: TTBEditItemOptions);
begin
  if FEditOptions <> Value then begin
    FEditOptions := Value;
    Change (True);
  end;
end;

procedure TTBEditItem.SetEditCaption (Value: String);
begin
  if FEditCaption <> Value then begin
    FEditCaption := Value;
    Change (True);
  end;
end;

procedure TTBEditItem.SetEditWidth (Value: Integer);
begin
  if FEditWidth <> Value then begin
    FEditWidth := Value;
    Change (True);
  end;
end;

procedure TTBEditItem.SetMaxLength (Value: Integer);
begin
  if FMaxLength <> Value then begin
    FMaxLength := Value;
    Change (True);
  end;
end;

procedure TTBEditItem.SetText (Value: String);
begin
  if FText <> Value then begin
    FText := Value;
    Change (True);
  end;
end;


{ TTBEditItemViewer }

procedure TTBEditItemViewer.EditWndProc (var Message: TMessage);
var
  Item: TTBEditItem;

  procedure AcceptText;
  var
    S: String;
    Accept: Boolean;
  begin
    S := FEditControl.Text;
    Accept := True;
    if Assigned(Item.FOnAcceptText) then
      Item.FOnAcceptText (Self, S, Accept);
    if Accept then
      Item.Text := S;
  end;

begin
  Item := TTBEditItem(Self.Item);
  if FEditControl = nil then
    Exit;
  if Message.Msg = WM_CHAR then
    case TWMChar(Message).CharCode of
      VK_TAB: begin
          FEditControlStatus := [ecsAccept];
          AcceptText;
          Exit;
        end;
      VK_RETURN: begin
          FEditControlStatus := [ecsAccept, ecsClose];
          AcceptText;
          Exit;
        end;
      VK_ESCAPE: begin
          FEditControlStatus := [];
          Exit;
        end;
    end;
  TEditAccess(FEditControl).WndProc (Message);
end;

procedure TTBEditItemViewer.GetEditRect (var R: TRect);
var
  Item: TTBEditItem;
  DC: HDC;
begin
  Item := TTBEditItem(Self.Item);
  DC := GetDC(0);
  try
    SelectObject (DC, ToolbarFont.Handle);
    R := BoundsRect;
    if not View.IsToolbar and (Item.EditCaption <> '') then begin
      Inc (R.Left, GetTextWidth(DC, Item.EditCaption, True) +
        EditMenuMidWidth + EditMenuTextMargin * 2);
    end;
  finally
    ReleaseDC (0, DC);
  end;
end;

procedure TTBEditItemViewer.CalcSize (const Canvas: TCanvas;
  var AWidth, AHeight: Integer);
var
  Item: TTBEditItem;
  DC: HDC;
  TextHeight, MinHeight: Integer;
begin
  Item := TTBEditItem(Self.Item);
  DC := Canvas.Handle;
  TextHeight := GetTextHeight(DC);
  AWidth := Item.FEditWidth;
  AHeight := TextHeight;
  if not IsToolbarStyle and (Item.EditCaption <> '') then begin
    Inc (AWidth, GetTextWidth(DC, Item.EditCaption, True) + EditMenuMidWidth +
      EditMenuTextMargin * 2);
  end;
  MinHeight := AHeight + (EditMenuTextMargin * 2) + 1;
  if not IsToolbarStyle then
    Inc (AHeight, DivRoundUp(AHeight, 4));
  if AHeight < MinHeight then
    AHeight := MinHeight;
end;

function TTBEditItemViewer.CaptionShown: Boolean;
begin
  Result := not IsToolbarStyle and inherited CaptionShown;
end;

function TTBEditItemViewer.GetCaptionText: String;
begin
  Result := TTBEditItem(Item).EditCaption;
end;

procedure TTBEditItemViewer.Draw (const Canvas: TCanvas;
  const ClientAreaRect: TRect; IsSelected, IsPushed, UseMenuColor: Boolean);
const
  FillColors: array[Boolean] of Integer = (COLOR_BTNFACE, COLOR_WINDOW);
  TextColors: array[Boolean] of Integer = (COLOR_GRAYTEXT, COLOR_WINDOWTEXT);
var
  Item: TTBEditItem;
  DrawDisabledShadow: Boolean;
  DC: HDC;
  S: String;
  R, R2: TRect;
  W: Integer;
  SaveFont: HFONT;
  SaveIndex: Integer;
begin
  Item := TTBEditItem(Self.Item);

  DrawDisabledShadow := not IsSelected or not UseMenuColor;

  DC := Canvas.Handle;
  SaveFont := SelectObject(DC, ToolbarFont.Handle);
  SetBkMode (DC, TRANSPARENT);

  R := ClientAreaRect;

  if not IsToolbarStyle and (Item.EditCaption <> '') then begin
    S := Item.EditCaption;
    SetBkColor (DC, GetSysColor(tbItemBkSysColors[UseMenuColor, IsSelected]));
    SetTextColor (DC, GetSysColor(tbItemTextSysColors[UseMenuColor, IsSelected]));
    W := GetTextWidth(DC, S, True) + EditMenuTextMargin * 2;
    R.Right := R.Left + W;
    if not IsSelected then
      FillRect (DC, R, GetSysColorBrush(tbItemBkSysColors[UseMenuColor, False]))
    else
      FillRect (DC, R, GetSysColorBrush(COLOR_HIGHLIGHT));
    Inc (R.Left, EditMenuTextMargin);
    DrawItemCaption (Canvas, R, S, not Item.Enabled and DrawDisabledShadow,
      DT_SINGLELINE or DT_LEFT or DT_VCENTER);
    R := ClientAreaRect;
    Inc (R.Left, W);

    R2 := R;
    Inc (R.Left, EditMenuMidWidth);
    SaveIndex := SaveDC(DC);
    ExcludeClipRect (DC, R.Left, R.Top, R.Right, R.Bottom);
    FillRect (DC, R2, GetSysColorBrush(tbItemBkSysColors[UseMenuColor, False]));
    RestoreDC (DC, SaveIndex);
  end;

  { Selected border }
  if not IsSelected or not Item.Enabled then begin
    FrameRect (DC, R, GetSysColorBrush(COLOR_BTNFACE));
    InflateRect (R, -1, -1);
    FrameRect (DC, R, GetSysColorBrush(FillColors[not Item.Enabled]));
  end
  else begin
    DrawEdge (DC, R, BDR_SUNKENOUTER, BF_RECT or BF_ADJUST);
    FrameRect (DC, R, GetSysColorBrush(COLOR_BTNFACE));
  end;
  InflateRect (R, -1, -1);
  { Fill }
  FillRect (DC, R, GetSysColorBrush(FillColors[Item.Enabled]));
  InflateRect (R, -1, -1);
  { Text }
  if Item.Text <> '' then begin
    S := Item.Text;
    SetBkColor (DC, GetSysColor(FillColors[Item.Enabled]));
    SetTextColor (DC, GetSysColor(TextColors[Item.Enabled]));
    DrawText (DC, PChar(S), Length(S), R, DT_SINGLELINE or DT_NOPREFIX);
  end;

  SelectObject (DC, SaveFont);
end;

procedure TTBEditItemViewer.GetCursor (const Pt: TPoint; var ACursor: HCURSOR);
var
  R: TRect;
begin
  if not Item.Enabled then
    Exit;
  GetEditRect (R);
  OffsetRect (R, -BoundsRect.Left, -BoundsRect.Top);
  InflateRect (R, -2, -2);
  if PtInRect(R, Pt) then
    ACursor := LoadCursor(0, IDC_IBEAM);
end;

function TTBEditItemViewer.EditLoop (const CapHandle: HWND): Boolean;

  procedure ControlMessageLoop;

    function PointInWindow (const Wnd: HWND; const P: TPoint): Boolean;
    var
      W: HWND;
    begin
      Result := False;
      W := WindowFromPoint(P);
      if W = 0 then Exit;
      if W = Wnd then
        Result := True
      else
        if IsChild(Wnd, W) then
          Result := True;
    end;

    function ContinueLoop: Boolean;
    begin
      Result := (ecsContinueLoop in FEditControlStatus) and
        FEditControl.Focused and Item.Enabled;
    end;

  var
    FirstLoop: Boolean;
    Msg: TMsg;
  begin
    try
      FirstLoop := True;
      while ContinueLoop do begin
        if not FirstLoop then
          WaitMessage;
        FirstLoop := False;
        while ContinueLoop and PeekMessage(Msg, 0, 0, 0, PM_NOREMOVE) do begin
          case Msg.message of
            WM_SYSKEYDOWN: begin
                if (Msg.wParam = VK_MENU) or (Msg.wParam = VK_F10) then begin
                  FEditControlStatus := [ecsClose];
                  Exit;
                end
              end;
            WM_LBUTTONDOWN, WM_RBUTTONDOWN, WM_MBUTTONDOWN,
            WM_NCLBUTTONDOWN, WM_NCRBUTTONDOWN, WM_NCMBUTTONDOWN: begin
                if Msg.hwnd <> FEditControl.Handle then
                  Exit;
              end;
            WM_MOUSEMOVE, WM_NCMOUSEMOVE: begin
                if GetCapture = CapHandle then begin
                  if PointInWindow(FEditControl.Handle, Msg.pt) then
                    ReleaseCapture;
                end
                else if GetCapture = 0 then begin
                  if not PointInWindow(FEditControl.Handle, Msg.pt) then
                    SetCapture (CapHandle);
                end;
                if GetCapture = CapHandle then
                  SetCursor (LoadCursor(0, IDC_ARROW));
              end;
          end;
          if PeekMessage(Msg, 0, 0, 0, PM_REMOVE or PM_NOYIELD) then begin
            if ((Msg.message = WM_MOUSEMOVE) or (Msg.message = WM_NCMOUSEMOVE)) and
               (Msg.hwnd = CapHandle) then
              { discard, so that the selection doesn't get changed }
            else begin
              TranslateMessage (Msg);
              DispatchMessage (Msg);
            end;
          end;
        end;
      end;
    finally
      { Make sure there are no outstanding WM_*CHAR messages }
      RemoveMessages (WM_CHAR, WM_DEADCHAR);
      RemoveMessages (WM_SYSCHAR, WM_SYSDEADCHAR);
    end;
  end;

var
  Item: TTBEditItem;
  R: TRect;
  ActiveWnd, FocusWnd: HWND;
begin
  Item := TTBEditItem(Self.Item);
  GetEditRect (R);
  if IsRectEmpty(R) then begin
    Result := False;
    Exit;
  end;

  ActiveWnd := GetActiveWindow;
  FocusWnd := GetFocus;

  { Create the edit control }
  InflateRect (R, -3, -3);
  //View.FreeNotification (Self);
  FEditControl := TEdit.Create(nil);
  try
    FEditControl.Name := Format('%s_edit_control_%p', [ClassName,
      Pointer(FEditControl)]);
    FEditControl.BorderStyle := bsNone;
    FEditControl.AutoSize := False;
    FEditControl.Font.Assign (ToolbarFont);
    FEditControl.Text := Item.Text;
    FEditControl.MaxLength := Item.FMaxLength;
    FEditControl.BoundsRect := R;
    FEditControl.ParentWindow := View.Window.Handle;
    FEditControl.WindowProc := EditWndProc;
    FEditControl.SelectAll;
    FEditControl.SetFocus;
    if GetActiveWindow <> ActiveWnd then
      { don't gray out title bar of old active window }
      SendMessage (ActiveWnd, WM_NCACTIVATE, 1, 0)
    else
      ActiveWnd := 0;

    FEditControlStatus := [ecsContinueLoop];
    ControlMessageLoop;
  finally
    FreeAndNil (FEditControl);
  end;

  { ensure the area underneath the edit control is repainted immediately }
  View.Window.Update;
  { If app is still active, set focus to previous control and restore capture
    to CapHandle if another control hasn't taken it }
  if GetActiveWindow <> 0 then begin
    SetFocus (FocusWnd);
    if GetCapture = 0 then
      SetCapture (CapHandle);
  end;
  if ActiveWnd <> 0 then
    SendMessage (ActiveWnd, WM_NCACTIVATE, Ord(GetActiveWindow = ActiveWnd), 0);

  Result := ecsClose in FEditControlStatus;
  if not Result and (GetCapture = CapHandle) then begin
    if ecsAccept in FEditControlStatus then
      { if we are accepting but not closing, Tab must have been pressed }
      View.Selected := View.NextSelectable(View.Selected,
        GetKeyState(VK_SHIFT) >= 0);
    if View.IsToolbar then
      View.EnterToolbarLoop ([tbetKeyboardControl]);
  end;
end;

procedure TTBEditItemViewer.BeginEdit (var CancelLoop: Boolean);
begin
  if Item.Enabled and EditLoop(CapHandle) then begin
    CancelLoop := True;
    if ecsAccept in FEditControlStatus then
      View.GivePriority (Self);
  end;
end;

procedure TTBEditItemViewer.KeyDown (var Key: Word; Shift: TShiftState;
  var EventData: TTBItemEventData);
begin
  if Key = VK_RETURN then begin
    Key := 0;
    BeginEdit (EventData.CancelLoop);
  end;
end;

procedure TTBEditItemViewer.MouseDown (Shift: TShiftState; X, Y: Integer;
  var EventData: TTBItemEventData);
begin
  BeginEdit (EventData.CancelLoop);
end;

procedure TTBEditItemViewer.MouseUp (X, Y: Integer;
  var EventData: TTBItemEventData);
begin
  BeginEdit (EventData.CancelLoop);
end;

end.
